/*
 * $Revision: 1.4 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2006/07/19 13:38:00 $ 
 ***************************************************************/
 
/** \file
 * \brief Provides a simple, dfs-based algorithm for
 * biconnectivity augmentation.
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/

#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_DFS_MAKE_BICONNECTED_H
#define OGDF_DFS_MAKE_BICONNECTED_H



#include <ogdf/module/AugmentationModule.h>

namespace ogdf {

/**
 * \brief Implementation of a DFS-based algorithm for biconnectivity augmentation.
 *
 * The class \a DfsMakeBiconnected implements an augmentation algorithms
 * that augments a graph to a biconnected graph. In addition, if the graph was
 * planar before augmentation, the resulting graph will be biconnected and
 * planar.
 * The algorithm simply uses DFS and, whenever a cut vertex is discovered,
 * a new edge is added.
 */

class DfsMakeBiconnected : public AugmentationModule {
public:
	// construction
	DfsMakeBiconnected() { }
	// destruction
	~DfsMakeBiconnected() { }

protected:
	void doCall(Graph& G, List<edge> &L);
};

} // end namespace ogdf

#endif
