/*
 * $Revision: 1.5 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2007/03/14 13:54:40 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of class TwoDHashIterator.
 *
 * This class implements an iterator for the TwoDHashArray.
 * 
 * \author Ren Weiskircher
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif


#ifndef OGDF_TWODHASHITERATOR_H
#define OGDF_TWODHASHITERATOR_H


namespace ogdf {


template< class I, class E > class TwoDHashArray;


template< class I, class E>
class TwoDHashIterator {

	friend class TwoDHashArray<I,E>;

	public:

		TwoDHashIterator() {};

		TwoDHashIterator(const TwoDHashIterator<I,E> &td) {

			m_it1 = td.m_it1;

			m_it2 = td.m_it2;

		}

		TwoDHashIterator<I,E> &operator=(const TwoDHashIterator<I,E> &td) {

			m_it1 = td.m_it1;

			m_it2 = td.m_it2;

			return *this;

		}

		//returns true if the iterator points to an element

		bool valid() const {

			// m_it2 can only be valid if m_it2 is valid, too

			// and if m_it2 is valid, than m_it1 is also valid

			if(m_it2.valid())

				return true;

			return false;

		}

		//returns the two keys of the current element in the arguments given

		void key(I& i1, I& i2) const {

			i1 = m_it1.key();

			i2 = m_it2.key();

		}

		//returns a const reference to the information stored in the

		//current element

		const E &info() const {

			return m_it2.info();

		}

		//sets the iterator to the next element in the hash array

		TwoDHashIterator<I,E> &operator++() {

			++m_it2;

			if(!m_it2.valid()) {

				++m_it1;

				if(m_it1.valid())

					m_it2 = m_it1.info().begin();

			}

			return *this;

		}

	private:

		HashConstIterator< I,HashingArray< I,E > > m_it1;

		//Itertor for iterating over the HashArrays

		HashConstIterator< I,E > m_it2; //Iterator of the current HashingArray

};

}



#endif

