/*
 * $Revision: 1.3 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2006/07/19 13:38:00 $ 
 ***************************************************************/
 
/** \file
 * \brief Mathematical Helpers
 * 
 * \author Markus Chimani
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/
 
#ifndef OGDF_MATH_H
#define OGDF_MATH_H

namespace ogdf {

#define DOUBLE_EPS 0.000001

class Math {
public:
	inline static int binomial(int n, int k) {
		if(k>n/2) k = n-k;
		int r = n;
		for(int i = 2; i<=k; ++i)
			r = (r * (n+1-i))/i;
		return r;
	}
	inline static double binomial(double n, double k) {
		if(k>n/2) k = n-k;
		double r = n;
		for(int i = 2; i<=k; ++i)
			r = (r * (n+1-i))/i;
		return r;
	}
	static int factorial(int n) {
		int r = 1;
		for(; n>1; --n) r *= n;
		return r;
	}

	static double factorial(double n) {
		double r = 1;
		for(; n>1; --n) r *= n;
		return r;
	}

	inline bool equald(double a, double b) {
		double d = a-b;
		return d < DOUBLE_EPS && d > -DOUBLE_EPS;
	}
};


}

#endif // OGDF_MATH_H
