/*
 * $Revision: 1.3 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2006/07/19 13:38:00 $ 
 ***************************************************************/
 
/** \file
 * \brief Cluster Planarity tests and Cluster Planar embedding
 * for C-connected Cluster Graphs
 * 
 * \author Sebastian Leipert
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_CCONNECT_CLUSTER_PLANAR_H
#define OGDF_CCONNECT_CLUSTER_PLANAR_H


#include <ogdf/internal/planarity/PlanarPQTree.h>
#include <ogdf/cluster/ClusterArray.h>
#include <ogdf/basic/EdgeArray.h>

namespace ogdf {

class CconnectClusterPlanar{

public:

	//aus CCCPE oder CCCP wieder entfernen
	enum ccErrorCode {none = 0, 
					  nonConnected = 1, 
					  nonCConnected = 2, 
					  nonPlanar = 3,
					  nonCPlanar = 4
	};
	ccErrorCode errCode() {return m_errorCode;}


	// Constructor
	CconnectClusterPlanar();

	// Destructor
	virtual ~CconnectClusterPlanar();

	// Tests if a ClusterGraph is C-planar
	virtual bool call(ClusterGraph &C);

	// Tests if a ClusterGraph is C-planar
	// Specifies reason for non planarity
	bool call(ClusterGraph &C, char (&code)[124]);

	// Tests if a const ClusterGraph is C-planar
	virtual bool call(const ClusterGraph &C);

private:

	bool planarityTest(ClusterGraph &C,
					   cluster &act,
					   Graph &G);

	bool preProcess(ClusterGraph &C,Graph &G);

	bool preparation(Graph  &G,cluster &C,node superSink);

	bool doTest(Graph &G,
				NodeArray<int> &numbering,
				cluster &cl,
				node superSink,
				EdgeArray<edge> &edgeTable);
	
	void prepareParallelEdges(Graph &G);


	void constructWheelGraph(ClusterGraph &C,
							 Graph &G,
							 cluster &parent,
							 PlanarPQTree* T,
							 EdgeArray<node> &outgoingTable);


	//private Members for handling parallel edges
	EdgeArray<ListPure<edge> >	m_parallelEdges;
	EdgeArray<bool>				m_isParallel;
	ClusterArray<PlanarPQTree*> m_clusterPQTree; 
	int	m_parallelCount;
	char errorCode[124];
	ccErrorCode m_errorCode;

	
};

} // end namespace ogdf


#endif
