/*
 * $Revision: 1.2 $
 * 
 * last checkin:
 *   $Author: chimani $ 
 *   $Date: 2005/11/15 12:40:06 $ 
 ***************************************************************/
 
/** \file
 * \brief Declares ClusterGraphCopyAttributes...
 * 
 * ...which manages access on copy of an attributed graph.
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_A_CLUSTER_GRAPH_COPY_H
#define OGDF_A_CLUSTER_GRAPH_COPY_H



#include <ogdf/layered/ExtendedNestingGraph.h>
#include <ogdf/cluster/ClusterGraphAttributes.h>


namespace ogdf {

//---------------------------------------------------------
// ClusterGraphCopyAttributes 
// manages access on copy of an attributed cluster graph
//---------------------------------------------------------
class ClusterGraphCopyAttributes {

	const ExtendedNestingGraph *m_pH;
	ClusterGraphAttributes     *m_pACG;
	NodeArray<double> m_x, m_y;

public:
	// construction
	ClusterGraphCopyAttributes(
		const ExtendedNestingGraph &H,
		ClusterGraphAttributes &ACG) :
		m_pH(&H), m_pACG(&ACG), m_x(H,0), m_y(H,0) { }

	// destruction
	~ClusterGraphCopyAttributes() { }

	const ClusterGraphAttributes &getClusterGraphAttributes() const { return *m_pACG; }

	// returns width of node v
	double getWidth(node v) const {
		node vOrig = m_pH->origNode(v);
		return (vOrig == 0) ? 0.0 : m_pACG->width(vOrig);
	}

	// returns height of node v
	double getHeight(node v) const {
		node vOrig = m_pH->origNode(v);
		return (vOrig == 0) ? 0.0 : m_pACG->height(vOrig);
	}

	// returns reference to x-coord. of node v
	const double &x(node v) const {
		return m_x[v];
	}

	// returns reference to x-coord. of node v
	double &x(node v) {
		return m_x[v];
	}

	// returns reference to y-coord. of node v
	const double &y(node v) const {
		return m_y[v];
	}

	// returns reference to y-coord. of node v
	double &y(node v) {
		return m_y[v];
	}

	double top(cluster cOrig) const {
		return m_pACG->clusterYPos(cOrig);
	}

	double bottom(cluster cOrig) const {
		return m_pACG->clusterYPos(cOrig) + m_pACG->clusterHeight(cOrig);
	}

	void setClusterRect(
		cluster cOrig,
		double left,
		double right,
		double top,
		double bottom)
	{
		m_pACG->clusterXPos  (cOrig) = left;
		m_pACG->clusterYPos  (cOrig) = top;
		m_pACG->clusterWidth (cOrig) = right-left;
		m_pACG->clusterHeight(cOrig) = bottom-top;
	}

	void setClusterLeftRight(
		cluster cOrig,
		double left,
		double right)
	{
		m_pACG->clusterXPos  (cOrig) = left;
		m_pACG->clusterWidth (cOrig) = right-left;
	}

	void setClusterTopBottom(
		cluster cOrig,
		double top,
		double bottom)
	{
		m_pACG->clusterYPos  (cOrig) = top;
		m_pACG->clusterHeight(cOrig) = bottom-top;
	}

	// sets attributes for the original graph in AG
	void transform();
};


} // end namespace ogdf


#endif
