/*
 * $Revision: 1.5 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2007/03/14 13:54:40 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of module to check orthogonal representations
 *        for turn-regularity and dissect faces to achieve
 *        turn-regularity.
 * 
 * \author Karsten Klein
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif


#ifndef OGDF_TRPROCESSOR_H
#define OGDF_TRPROCESSOR_H


#include <ogdf/orthogonal/OrthoRep.h>

namespace ogdf {


class TRProcessor
{
public:

	TRProcessor(PlanRepUML& PG, OrthoRep& OR);
    TRProcessor();

    //returns true if the given shape in OR is turn-regular
	bool isTurnRegular();

	//achieve turn-regularity by dissecting the representation
	//using given heuristics and given direction
    void dissect(const int heuristicsnr, OrthoDir od);

	//split to nearest/farest kitty Corner
	void setSplitNear(bool b) {m_splitNear = b;}
	bool getSplitNear() {return m_splitNear;}

	//number of inserted edges
	int getSplitCount() {return m_splitCount;}

protected:
	//returns true if face f is turnregular
	bool isTurnRegular(face f, bool isExternal);

private:
	bool m_splitNear; //decide if non-tr faces are splitted to nearest kitty corner
	//todo use ints to allow arbitrary strategies

	//number of inserted edges in dissection
	int m_splitCount;

};//class TRProcessor


} // end namespace ogdf


#endif
