/*
 * $Revision: 1.3 $
 * 
 * last checkin:
 *   $Author: chimani $ 
 *   $Date: 2005/11/15 12:40:05 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of class DfsAcyclicSubgraph
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/

#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_DFS_ACYCLIC_SUBGRAPH_H
#define OGDF_DFS_ACYCLIC_SUBGRAPH_H



#include <ogdf/module/AcyclicSubgraphModule.h>


namespace ogdf {

class GraphAttributes;



//! computes a maximal acyclic subgraph 
/**
 * this is done by deleting all DFS backedges (linear-time)
 */
class DfsAcyclicSubgraph : public AcyclicSubgraphModule {
public:
	//! computes set of edges arcSet, which have to be deleted in acyclic subgraph
	void call (const Graph &G, List<edge> &arcSet);

	//! call for UML graph
	/**
	 *  computes set of edges arcSet, which have to be deleted
	 * in acyclic subgraph
	 */
	void callUML (const GraphAttributes &AG, List<edge> &arcSet);

private:
	int dfsFindHierarchies(
		const GraphAttributes &AG,
		NodeArray<int> &hierarchy,
		int i,
		node v);

	void dfsBackedgesHierarchies(
		const GraphAttributes &AG,
		node v,
		NodeArray<int> &number,
		NodeArray<int> &completion,
		int &nNumber,
		int &nCompletion);

};


} // end namespace ogdf


#endif
