/*
 * $Revision: 1.3 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2005/11/18 15:25:27 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of class GeedyCycleRemoval.
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_GREEDY_CYCLE_REMOVAL_H
#define OGDF_GREEDY_CYCLE_REMOVAL_H



#include <ogdf/module/AcyclicSubgraphModule.h>
#include <ogdf/basic/NodeArray.h>


namespace ogdf {


//---------------------------------------------------------
// GreedyCycleRemoval
// greedy heuristic for computing a maximal acyclic
// subgraph (linear-time)
//---------------------------------------------------------
class GreedyCycleRemoval : public AcyclicSubgraphModule {
public:
	// computes set of edges arcSet, which have to be deleted
	// in acyclic subgraph
	void call (const Graph &G, List<edge> &arcSet);

private:
	void dfs (node v, const Graph &G);

	int m_min, m_max, m_counter;

	NodeArray<int> m_in, m_out, m_index;
	Array<ListPure<node> > m_B;
	NodeArray<ListIterator<node> > m_item;
	NodeArray<bool> m_visited;
};


} // end namespace ogdf


#endif
