/*
 * $Revision: 1.3 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2006/07/19 13:38:00 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration and implementation of Level class
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_LEVEL_H
#define OGDF_LEVEL_H



#include <ogdf/basic/Graph.h>
#include <ogdf/basic/SList.h>
#include <ogdf/basic/tuples.h>


namespace ogdf {

class Hierarchy;


//---------------------------------------------------------
// Level
// representation of levels in hierarchies
//---------------------------------------------------------
class Level {

	friend class Hierarchy;

	Array<node> m_nodes;     // nodes on layer
	Hierarchy *m_pHierarchy; // hierarchy to which level belongs
	int m_index; // index of level

public:
	// construction
	Level(Hierarchy *pHierarchy, int index, int num) :
		m_nodes(num), m_pHierarchy(pHierarchy), m_index(index) { }

	// destruction
	~Level() { }

	// returns node on position i
	const node &operator[](int i) const { return m_nodes[i]; }
	// returns node on position i
	node &operator[](int i) { return m_nodes[i]; }

	// returns number of nodes on level
	int size() const { return m_nodes.size(); }
	// returns maximal array index (= size()-1)
	int high() const { return m_nodes.high(); }

	// returns array index of level in hierarchy
	int index() const { return m_index; }

	// returns (sorted) array of adjacent nodes (according to direction())
	const Array<node> &adjNodes(node v);

	// returns hierarchy to which level belongs
	const Hierarchy &hierarchy() const { return *m_pHierarchy; }

	// exchanges nodes at position i and j
	void swap(int i, int j);

	// sorts nodes according to weight using quicksort
	void sort(NodeArray<double> &weight);
	// sorts nodes according to weight using bucket sort
	void sort(NodeArray<int> &weight, int minBucket, int maxBucket);

	void recalcPos();

	friend ostream &operator<<(ostream &os, const Level &L) {
		os << L.m_nodes;
		return os;
	}

private:
	void getIsolatedNodes(SListPure<Tuple2<node,int> > &isolated);
	void setIsolatedNodes(SListPure<Tuple2<node,int> > &isolated);

	OGDF_MALLOC_NEW_DELETE
};

} // end namespace ogdf


#endif
