/*
 * $Revision: 1.3 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2006/07/19 13:38:00 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of hierachrical ranking algorithm
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_LONGEST_PATH_RANKING_H
#define OGDF_LONGEST_PATH_RANKING_H



#include <ogdf/module/RankingModule.h>
#include <ogdf/module/AcyclicSubgraphModule.h>
#include <ogdf/basic/ModuleOption.h>
#include <ogdf/basic/SList.h>
#include <ogdf/basic/tuples.h>
#include <ogdf/basic/NodeArray.h>


namespace ogdf {

	class GraphCopySimple;
	class GraphAttributes;

//---------------------------------------------------------
// LongestPathRanking
// linear-time node ranking for hierarchical graphs
//---------------------------------------------------------
class LongestPathRanking : public RankingModule {

	ModuleOption<AcyclicSubgraphModule> m_subgraph; // option for acyclic sugraph
	bool m_sepDeg0; // option: separate layer for isolated nodes
	bool m_separateMultiEdges;
	bool m_alignBaseClasses;
	bool m_alignSiblings;

	int m_offset, m_maxN;

	NodeArray<bool> m_isSource, m_finished;
	NodeArray<SListPure<Tuple2<node,int> > > m_adjacent;
	NodeArray<int> m_ingoing;

public:
	// construction
	LongestPathRanking();

	// computes node ranking of G in rank
	void call(const Graph &G, NodeArray<int> &rank);

	// computes node ranking of G in rank
	void call(const Graph &G, const EdgeArray<int> &length, NodeArray<int> &rank);

	// call for UML graphs with special treatement for
	// inheritance hierarchies
	void callUML(const GraphAttributes &AG, NodeArray<int> &rank);

	// option determining if nodes with degree 0 are placed
	// on a separate layer (default is true)
	bool separateDeg0Layer() const { return m_sepDeg0; }
	void separateDeg0Layer (bool sdl) { m_sepDeg0 = sdl; }

	// option determining if adjacent nodes of multi-edges have a rank
	// difference of at least two
	bool separateMultiEdges() const { return m_separateMultiEdges; }
	void separateMultiEdges(bool b) { m_separateMultiEdges = b; }

	// option determining if nodes representing base classes
	// are aligned at the top
	bool alignBaseClasses() const { return m_alignBaseClasses; }
	void alignBaseClasses(bool b) { m_alignBaseClasses = b; }

	// option determining if nodes representing base classes
	// are aligned at the top
	bool alignSiblings() const { return m_alignSiblings; }
	void alignSiblings(bool b) { m_alignSiblings = b; }

	// module option for acyclic subgraph algorithm
	// (default is DfsAcyclicSubgraph)
	void setSubgraph(AcyclicSubgraphModule *pSubgraph) {
		m_subgraph.set(pSubgraph);
	}

private:
	void doCall(const Graph& G,
		NodeArray<int> &rank,
		EdgeArray<bool> &reversed,
		const EdgeArray<int> &length);

	void join(
		GraphCopySimple &GC,
		NodeArray<node> &superNode,
		NodeArray<SListPure<node> > &joinedNodes,
		node v, node w);

	void dfs(node v);
	void getTmpRank(node v, NodeArray<int> &rank);
	void dfsAdd(node v, NodeArray<int> &rank);
};


} // end namespace ogdf


#endif
