/*
 * $Revision: 1.3 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2006/07/19 13:38:00 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of class MedianHeuristic
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_MEDIAN_HEURISTIC_H
#define OGDF_MEDIAN_HEURISTIC_H



#include <ogdf/module/TwoLayerCrossMin.h>


namespace ogdf {


//---------------------------------------------------------
// MedianHeuristic
// implements the median heuristic for 2-layer 
// crossing minimization
//---------------------------------------------------------
class MedianHeuristic : public TwoLayerCrossMin {
public:

	// initializes crossing minimization
	// (must be called before any call's)
	void init (const Hierarchy &H) { m_weight.init(H); }

	// calls median heuristic for level L
	void call (Level &L);

	// does clean-up
	// (must be called after all call's)
	void cleanup () { m_weight.init(); }

private:
	NodeArray<int> m_weight; // median weight of node
};


} // end namespace ogdf


#endif
