/*
 * $Revision: 1.5 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2007/03/12 16:02:59 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of the optimal third phase of the sugiyama
 *        algorithm for clusters.
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_OPTIMAL_HIERARCHY_CLUSTER_LAYOUT_H
#define OGDF_OPTIMAL_HIERARCHY_CLUSTER_LAYOUT_H



#include <ogdf/module/HierarchyClusterLayoutModule.h>
#include <ogdf/basic/tuples.h>


namespace ogdf {


class OptimalHierarchyClusterLayout : public HierarchyClusterLayoutModule
{
#ifndef OGDF_LP_SOLVER
protected:
	void doCall(const ExtendedNestingGraph& H,ClusterGraphCopyAttributes &ACGC) {
		THROW_PARAM(LibraryNotSupportedException, lnscCoin);
	}

#else

public:
	// default constructor
	OptimalHierarchyClusterLayout();

	// copy constructor
	OptimalHierarchyClusterLayout(const OptimalHierarchyClusterLayout &);

	// destructor
	~OptimalHierarchyClusterLayout() { }


	// assignment
	OptimalHierarchyClusterLayout &operator=(const OptimalHierarchyClusterLayout &);


	//
	// Options
	//

	// Minimum x-distance between nodes on a layer
	double nodeDistance() const {
		return m_nodeDistance;
	}

	void nodeDistance(double x) {
		if(x >= 0)
			m_nodeDistance = x;
	}

	// Minimum y-distance between layers
	double layerDistance() const {
		return m_layerDistance;
	}

	void layerDistance(double x) {
		if(x >= 0)
			m_layerDistance = x;
	}

	// Distance between layers is fixed or adjusted
	bool fixedLayerDistance() const {
		return m_fixedLayerDistance;
	}

	void fixedLayerDistance(bool b) {
		m_fixedLayerDistance = b;
	}

	// Weight of edge segments connecting to vertical segments
	double weightSegments() const {
		return m_weightSegments;
	}

	void weightSegments(double w) {
		if(w > 0.0 && w <= 100.0)
			m_weightSegments = w;
	}

	// Weight for balancing variables; 0.0 means no balancing
	double weightBalancing() const {
		return m_weightBalancing;
	}

	void weightBalancing(double w) {
		if(w >= 0.0 && w <= 100.0)
			m_weightBalancing = w;
	}

	// Weight for cluster boundary variables
	double weightClusters() const {
		return m_weightClusters;
	}

	void weightClusters(double w) {
		if(w > 0.0 && w <= 100.0)
			m_weightClusters = w;
	}


protected:
	void doCall(const ExtendedNestingGraph& H,ClusterGraphCopyAttributes &ACGC);

private:
	void buildLayerList(
		const LHTreeNode *vNode,
		List<Tuple2<int,double> > &L);

	void computeXCoordinates(
		const ExtendedNestingGraph& H,
		ClusterGraphCopyAttributes &AGC);
	void computeYCoordinates(
		const ExtendedNestingGraph& H,
		ClusterGraphCopyAttributes &AGC);

	// options
	double m_nodeDistance;
	double m_layerDistance;
	bool   m_fixedLayerDistance;

	double m_weightSegments;
	double m_weightBalancing;
	double m_weightClusters;

	// auxiliary data
	ClusterGraphCopyAttributes *m_pACGC;
	const ExtendedNestingGraph *m_pH;

	int m_vertexOffset;
	int m_segmentOffset;
	int m_clusterLeftOffset;
	int m_clusterRightOffset;

	NodeArray<bool>   m_isVirtual;
	NodeArray<int>    m_vIndex;
	ClusterArray<int> m_cIndex;

#endif
};

}


#endif
