/*
 * $Revision: 1.5 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2007/03/12 16:02:59 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of optimal ranking algorithm for Sugiyama
 *        algorithm.
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_OPTIMAL_RANKING_H
#define OGDF_OPTIMAL_RANKING_H



#include <ogdf/module/RankingModule.h>
#include <ogdf/module/AcyclicSubgraphModule.h>
#include <ogdf/basic/ModuleOption.h>
#include <ogdf/basic/NodeArray.h>


namespace ogdf {

//---------------------------------------------------------
// LongestPathRanking
// linear-time node ranking for hierarchical graphs
//---------------------------------------------------------
class OptimalRanking : public RankingModule {

	ModuleOption<AcyclicSubgraphModule> m_subgraph; // option for acyclic sugraph
	bool m_separateMultiEdges;

public:
	// construction
	OptimalRanking();

	// computes node ranking of G in rank
	void call(const Graph &G, NodeArray<int> &rank);

	// computes node ranking of G for given minimal edge lengths in rank
	void call(const Graph &G, const EdgeArray<int> &length, NodeArray<int> &rank);

	// computes node ranking of G for given edge costs and minimal edge lengths in rank
	void call(
		const Graph &G,
		const EdgeArray<int> &length,
		const EdgeArray<int> &cost,
		NodeArray<int> &rank);

	// option determining if adjacent nodes of multi-edges have a rank
	// difference of at least two
	bool separateMultiEdges() const { return m_separateMultiEdges; }
	void separateMultiEdges(bool b) { m_separateMultiEdges = b; }


	// module option for acyclic subgraph algorithm
	// (default is DfsAcyclicSubgraph)
	void setSubgraph(AcyclicSubgraphModule *pSubgraph) {
		m_subgraph.set(pSubgraph);
	}

private:
	void doCall(const Graph& G,
		NodeArray<int> &rank,
		EdgeArray<bool> &reversed,
		const EdgeArray<int> &length,
		const EdgeArray<int> &cost);
};


} // end namespace ogdf


#endif
