/*
 * $Revision: 1.4 $
 * 
 * last checkin:
 *   $Author: schulz $ 
 *   $Date: 2007/04/23 17:12:47 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of class SplitHeuristic.
 * 
 * \author Andrea Wagner
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/

#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_SPLIT_HEURISTIC_H
#define OGDF_SPLIT_HEURISTIC_H


#include <ogdf/basic/EdgeArray.h>
#include <ogdf/layered/CrossingsMatrix.h>
#include <ogdf/simultaneous/TwoLayerCrossMinSimDraw.h>

namespace ogdf
{

//---------------------------------------------------------
// SplitHeuristic
// implements the split heuristic for 2-layer 
// crossing minimization
//---------------------------------------------------------
class SplitHeuristic : public TwoLayerCrossMinSimDraw {
    public:
      void init (const Hierarchy &H);

      //! ordinary call
      /** 
       * implements the virtual method of the base class TwoLayerCrossMin
       */
      void call (Level &L);

      //! SimDraw call
      /**
       * implements the virtual method of the base class TwoLayerCrossMinSimDraw
       */
      void call (Level &L, const EdgeArray<unsigned int> *edgeSubGraph); 

      void cleanup ();

    private:
      CrossingsMatrix *m_cm;
      Array<node> buffer;

      void recCall(Level&, int low, int high);
    };

}// end namespace ogdf

#endif
