/*
 * $Revision: 1.4 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2006/07/19 13:38:01 $ 
 ***************************************************************/
 
/** \file
 * \brief Declares class CircularLayout
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/

#ifdef _MSC_VER
#pragma once
#endif


#ifndef OGDF_CIRCULAR_LAYOUT_H
#define OGDF_CIRCULAR_LAYOUT_H


#include <ogdf/module/LayoutModule.h>


namespace ogdf {

class GraphCopyAttributes;
struct ClusterStructure;

/**
 * \brief The circular layout algorithm.
 */
class CircularLayout : public LayoutModule
{
public:
	/** @{
	 * \brief Creates an instance of circular layout.
	 */
	CircularLayout();

	// destructor
	~CircularLayout() { }


	/** @} @{
	 * \brief Computes a circular layout of graph \a AG.
	 */
	void call(GraphAttributes &AG);

	/** @} @{
	 * \brief Returns the option <i>minimum circle distance</i>.
	 */
    const double minDistCircle() { return m_minDistCircle; }

	/**
	 * \brief Sets the option <i>minimum circle distance</i> to \a x.
	 */
    void minDistCircle(double x) { m_minDistCircle = x; }

	/** @} @{
	 * \brief Returns the option <i>minimum level distance</i>.
	 */
    const double minDistLevel() { return m_minDistLevel; }

	/**
	 * \brief Sets the option <i>minimum level distance</i> to \a x.
	 */
    void minDistLevel(double x) { m_minDistLevel = x; }

	/** @} @{
	 * \brief Returns the option <i>minimum sibling distance</i>.
	 */
    const double minDistSibling() { return m_minDistSibling; }

	/**
	 * \brief Sets the option <i>minimum sibling distance</i> to \a x.
	 */
    void minDistSibling(double x) { m_minDistSibling = x; }

	/** @} @{
	 * \brief Returns the option <i>minimum connected component distance</i>.
	 */
    const double minDistCC() { return m_minDistCC; }

	/**
	 * \brief Sets the option <i>minimum connected component distance</i> to \a x.
	 */
    void minDistCC(double x) { m_minDistCC = x; }

	/** @} @{
	 * \brief Returns the option <i>page ratio</i>.
	 */
    const double pageRatio() { return m_pageRatio; }

	/**
	 * \brief Sets the option <i>page ratio</i> to \a x.
	 */
    void pageRatio(double x) { m_pageRatio = x; }

	/** @} */

private:
	double m_minDistCircle;  //!< The minimal distance between nodes on a circle.
	double m_minDistLevel;   //!< The minimal distance between father and child circle.
	double m_minDistSibling; //!< The minimal distance between circles on same level.
	double m_minDistCC;      //!< The minimal distance between connected components.
	double m_pageRatio;

	void doCall(GraphCopyAttributes &AG, ClusterStructure &C);

	void assignClustersByBiconnectedComponents(ClusterStructure &C);

	int sizeBC(node vB);

	void computePreferedAngles(
		ClusterStructure &C,
		const Array<double> &outerRadius,
		Array<double> &preferedAngle);

	void assignPrefAngle(ClusterStructure &C,
		const Array<double> &outerRadius,
		Array<double> &preferedAngle,
		int c,
		int l,
		double r1);

};//CircularLayout


} // end namespace ogdf


#endif
