/** \file
 * \brief Declaration of interface for embedder for
 * graphs.
 * 
 * \author Thorsten Kerkhof (thorsten.kerkhof@udo.edu)
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/

#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_EMBEDDER_MODULE_H
#define OGDF_EMBEDDER_MODULE_H

#include <ogdf/planarity/PlanRep.h>
#include <ogdf/basic/Module.h>
#include <ogdf/basic/Logger.h>
#include <ogdf/basic/Timeouter.h>

namespace ogdf {

//---------------------------------------------------------
// EmbedderModule
// base class of embedder algorithms
//---------------------------------------------------------
class EmbedderModule : public Module, public Timeouter {
public:
	// construction / destruction
	EmbedderModule() { }
	virtual ~EmbedderModule() { }

	/**
   * \brief Call embedder algorithm.
   * \param G is the original graph. Its adjacency list has to be  changed by the embedder.
	 * \param adjExternal is an adjacency entry on the external face and has to be set by the embedder.
   */
	virtual void call(PlanRep& PG, adjEntry& adjExternal) = 0;

	/**
   * \brief Call embedder algorithm.
   * \param G is the original graph. Its adjacency list has to be  changed by the embedder.
	 * \param adjExternal is an adjacency entry on the external face and has to be set by the embedder.
   */
	void operator()(PlanRep& PG, adjEntry& adjExternal) { call(PG, adjExternal); }

	OGDF_MALLOC_NEW_DELETE
};

} // end namespace ogdf

#endif
