/*
 * $Revision: 1.4 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2007/03/12 16:02:59 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of interface for layout algorithms (class
 *        LayoutModule)
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_LAYOUT_MODULE_H
#define OGDF_LAYOUT_MODULE_H



#include <ogdf/basic/GraphAttributes.h>

namespace ogdf {


/**
 * \brief Interface of general layout algorithms.
 *
 */
class LayoutModule {
public:
	//! Initializes a layout module.
	LayoutModule() { }

	virtual ~LayoutModule() { }

	/**
	 * \brief Computes a layout of graph \a AG.
	 *
	 * This method is the actual algorithm call and must be implemented by
	 * derived classes.
	 * @param AG is the input graph and will also be assigned the layout information.
	 */
	virtual void call(GraphAttributes &AG) = 0;

	/**
	 * \brief Computes a layout of graph \a AG.
	 *
	 * @param AG is the input graph and will also be assigned the layout information.
	 */
	void operator()(GraphAttributes &AG) { call(AG); }

	OGDF_MALLOC_NEW_DELETE
};


} // end namespace ogdf


#endif
