/*
 * $Revision: 1.4 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2007/03/15 10:08:46 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of interface for planar layout algorithms for
 *        UML diagrams (used in planarization approach).
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_UML_PLANAR_LAYOUT_MODULE_H
#define OGDF_UML_PLANAR_LAYOUT_MODULE_H



#include <ogdf/planarity/PlanRepUML.h>
#include <ogdf/basic/Layout.h>



namespace ogdf {

	
	enum UMLOpt {umlOpAlign = 0x0001, umlOpScale = 0x0002, umlOpProg = 0x0004};


//---------------------------------------------------------
// UMLPlanarLayoutModule
// base class of planar UML layout algorithms
//---------------------------------------------------------
class UMLPlanarLayoutModule {
public:
	// construction / destruction
	UMLPlanarLayoutModule() { }
	virtual ~UMLPlanarLayoutModule() { }

	// call planar UML layout algorithm. Input is a planarized representation
	// PG, which may be modified, output is a layout of the (modified)
	// planarized representation in drawing
	// must be overridden by derived classes
	virtual void call(PlanRepUML &PG,
		adjEntry adjExternal,
		Layout &drawing) = 0;

	// call planar UML layout algorithm. Input is a planarized representation
	// PG, which may be modified, output is a layout of the (modified)
	// planarized representation in drawing
	void operator()(PlanRepUML &PG, adjEntry adjExternal, Layout &drawing) {
		call(PG,adjExternal,drawing);
	}

	const DPoint &getBoundingBox() const {
		return m_boundingBox;
	}

	//set options (generic, derived classes have to cope with the interpretation
	virtual void setOptions(int /* optionField */) { } //don't make it abstract
	virtual int getOptions() { return 0; } //don't make it abstract

	//object separation distance value
	// the minimum distance between edges and vertices
	virtual double separation() const = 0;

	virtual void separation(double sep) = 0;

protected:
	// stores the width and height of the computed layout
	// MUST BE SET BY ALGORITHM!
	DPoint m_boundingBox;

	// an algorithm can call setBoundingBox() for setting the
	// m_boundingBox variable (if no faster implementation is available)
	void setBoundingBox(PlanRepUML &PG, Layout &drawing);

	OGDF_MALLOC_NEW_DELETE
};


} // end namespace ogdf


#endif
