/*
 * $Revision: 1.5 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2007/03/14 13:54:40 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of interface for upward planar subgraph
 *        algorithms.
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_UPWARD_PLANAR_SUBGRAPH_MODULE_H
#define OGDF_UPWARD_PLANAR_SUBGRAPH_MODULE_H



#include <ogdf/basic/GraphCopy.h>


namespace ogdf {

//---------------------------------------------------------
// UpwardPlanarSubgraphModule
// base class of algorithms for computing an upward planar
// subgraph
//---------------------------------------------------------
class UpwardPlanarSubgraphModule {
public:
	// construction
	UpwardPlanarSubgraphModule() { }
	// destruction
	virtual ~UpwardPlanarSubgraphModule() { }

	// computes set of edges delEdges, which have to be deleted
	// in order to get an upward planar subgraph;
	// must be implemented by derived classes!
	virtual void call(const Graph &G, List<edge> &delEdges) = 0;


	// computes set of edges delEdges, which have to be deleted
	// in order to get an upward planar subgraph;
	void operator()(const Graph &G, List<edge> &delEdges) {
		call(G,delEdges);
	}


	// makes GC upward planar by deleting edges
	void callAndDelete(GraphCopy &GC, List<edge> &delOrigEdges);



	OGDF_MALLOC_NEW_DELETE
}; // class UpwardPlanarSubgraph



} // end namespace ogdf

#endif
