/*
 * $Revision: 1.5 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2007/03/12 16:02:59 $ 
 ***************************************************************/
 
/** \file
 * \brief Computes the orthogonal representation of a planar 
 *        representation of a UML graph.
 * 
 * \author Sebastian Leipert
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif


#ifndef OGDF_ORTHO_FORMER_UML_H
#define OGDF_ORTHO_FORMER_UML_H

#include <ogdf/orthogonal/OrthoRep.h>
#include <ogdf/planarity/PlanRepUML.h>

namespace ogdf {

class OrthoFormerUML
{

public:
	// construction
	OrthoFormerUML() {
		m_distributeEdges = true;
		m_multiAlign      = true;
	}

	// destruction
	~OrthoFormerUML() { }


	// Given a planar representation for a UML graph and its planar 
	// combinatorial embedding, call produces an orthogonal 
	// representation using Tamassias bend minimization algorithm.
	// A maximum number of bends per edge can be specified in 
	// startBoundBendsPerEdge. If the algorithm is not successful in 
	// producing a bend minimal representation subject to 
	// startBoundBendsPerEdge, it successively enhances the bound by 
	// one trying to compute an orthogonal representation.
	//
	// Using startBoundBendsPerEdge may not produce a bend minimal
	// representation in general.
	void call(PlanRepUML &PG,
			  CombinatorialEmbedding &E,
			  OrthoRep &OR,
			  int startBoundBendsPerEdge = 0);

	// returns option distributeEdges
    bool distributeEdges() { return m_distributeEdges; }
	// sets option distributeEdges to b
	void distributeEdges(bool b) { m_distributeEdges = b; }

	// returns option multiAlign
    bool multiAlign() { return m_multiAlign; }
	// sets option multiAlign to b
	void multiAlign(bool b) { m_multiAlign = b; }

private:
	bool m_distributeEdges; // distribute edges among all sides if degree > 4
    bool m_multiAlign;      // multi edges aligned on the same side

};


} // end namespace ogdf


#endif
