/*
 * $Revision: 1.5 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2007/03/14 13:54:40 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of class TileToRowsCCPacker.
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_TILE_TO_ROWS_CC_PACKER_H
#define OGDF_TILE_TO_ROWS_CC_PACKER_H



#include <ogdf/module/CCLayoutPackModule.h>



namespace ogdf {


//---------------------------------------------------------
// TileToRowsCCPacker
// implements simple packing algorithm for the layout of
// connected components
//---------------------------------------------------------
	class TileToRowsCCPacker : public CCLayoutPackModule {

	template<class POINT> struct RowInfo;

public:
	// construction / destruction
	TileToRowsCCPacker() { }
	virtual ~TileToRowsCCPacker() { }

	// call packing algorithm. Input is an array of boxes (for each box is
	// given its width and height); output is an array of offsets, i.e.,
	// the lower left coordinate of the respective box.
	void call(Array<DPoint> &box,
		Array<DPoint> &offset,
		double pageRatio = 1.0);

	// call packing algorithm. Input is an array of boxes (for each box is
	// given its width and height); output is an array of offsets, i.e.,
	// the lower left coordinate of the respective box.
	void call(Array<IPoint> &box,
		Array<IPoint> &offset,
		double pageRatio = 1.0);

private:
	template<class POINT>
	static void callGeneric(Array<POINT> &box,
		Array<POINT> &offset,
		double pageRatio);

	template<class POINT>
	static int findBestRow(Array<RowInfo<POINT> > &row,
		int nRows,
		double pageRatio,
		const POINT &rect);

};


} // end namespace ogdf


#endif
