/*
 * $Revision: 1.5 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2007/03/07 14:07:40 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of planarization with grid layout.
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_PLANARIZATION_GRID_LAYOUT_H
#define OGDF_PLANARIZATION_GRID_LAYOUT_H



#include <ogdf/module/GridLayoutModule.h>
#include <ogdf/basic/ModuleOption.h>
#include <ogdf/module/PlanarSubgraphModule.h>
#include <ogdf/module/EdgeInsertionModule.h>
#include <ogdf/module/GridLayoutModule.h>
#include <ogdf/module/CCLayoutPackModule.h>


namespace ogdf {


class PlanarizationGridLayout : public GridLayoutModule
{
public:
	PlanarizationGridLayout();

	~PlanarizationGridLayout() { }

	// returns current page ratio (= desired width / height of layout)
	double pageRatio() const {
		return m_pageRatio;
	}

	// sets page ratio
	void pageRatio(double ratio) {
		m_pageRatio = ratio;
	}

	// module option for the computation of the planar subgraph
	void setSubgraph(PlanarSubgraphModule *pSubgraph) {
		m_subgraph.set(pSubgraph);
	}

	// module option for the edge insertion module
	void setInserter(EdgeInsertionModule *pInserter) {
		m_inserter.set(pInserter);
	}

	// module option for the planar layout algorithm
	void setPlanarLayouter(GridLayoutPlanRepModule *pPlanarLayouter) {
		m_planarLayouter.set(pPlanarLayouter);
	}

	// module option for the arrangement of connected components
	void setPacker(CCLayoutPackModule *pPacker) {
		m_packer.set(pPacker);
	}


protected:
	//! Implements the algorithm call.
	void doCall(const Graph &G, GridLayout &gridLayout, IPoint &boundingBox);


private:
	ModuleOption<PlanarSubgraphModule>    m_subgraph;
	ModuleOption<EdgeInsertionModule>     m_inserter;
	ModuleOption<GridLayoutPlanRepModule> m_planarLayouter;
	ModuleOption<CCLayoutPackModule>      m_packer;

	double m_pageRatio;

	int m_nCrossings;
};


} // end namespace ogdf


#endif
