/*
 * $Revision: 1.5 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2007/03/14 13:54:40 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of class SubgraphPlanarizer.
 * 
 * \author Markus Chimani
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifndef OGDF_SUBGRAPH_PLANARIZER_H
#define OGDF_SUBGRAPH_PLANARIZER_H

#include <ogdf/module/CrossingMinimizationModule.h>
#include <ogdf/module/PlanarSubgraphModule.h>
#include <ogdf/module/EdgeInsertionModule.h>
#include <ogdf/basic/ModuleOption.h>
#include <ogdf/basic/Logger.h>


namespace ogdf
{

class SubgraphPlanarizer : public CrossingMinimizationModule, public Logger
{
	class CrossingStructure
	{
	public:
		CrossingStructure() : m_numCrossings(0) { }
		void init(PlanRep &PG, int weightedCrossingNumber);
		void restore(PlanRep &PG, int cc);
		
		int numberOfCrossings() const { return m_numCrossings; }
		int weightedCrossingNumber() const { return m_weightedCrossingNumber; }
		const SListPure<int> &crossings(edge e) const { return m_crossings[e]; }
		
	private:
		int m_numCrossings;
		int m_weightedCrossingNumber;
		EdgeArray<SListPure<int> > m_crossings;
	};

protected:	
	virtual ReturnType doCall(PlanRep &PG,
		int cc,
		const EdgeArray<int>  &cost,
		const EdgeArray<bool> &forbid,
		const EdgeArray<unsigned int>  &subgraphs,
		int& crossingNumber);

public:
	SubgraphPlanarizer();

	// module option for the computation of the planar subgraph
	void setSubgraph(PlanarSubgraphModule *pSubgraph) {
		m_subgraph.set(pSubgraph);
	}

	// module option for the edge insertion module
	void setInserter(EdgeInsertionModule *pInserter) {
		m_inserter.set(pInserter);
	}

	int permutations() { return m_permutations; }
	void permutations(int p) { m_permutations = p; }

	bool setTimeout() { return m_setTimeout; }
	void setTimeout(bool b) { m_setTimeout = b; }
	
private:
	ModuleOption<PlanarSubgraphModule>  m_subgraph;
	ModuleOption<EdgeInsertionModule>   m_inserter;

	int m_permutations;	
	bool m_setTimeout;	
};

}

#endif // OGDF_SUBGRAPH_PLANARIZER_H
