/*
 * $Revision: 1.5 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2007/03/14 09:35:58 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of class PlanarStraightLayout which represents
 *        a planar straight-line drawing algorithm.
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif


#ifndef OGDF_PLANAR_STRAIGHT_LAYOUT_H
#define OGDF_PLANAR_STRAIGHT_LAYOUT_H


#include <ogdf/module/GridLayoutModule.h>
#include <ogdf/basic/ModuleOption.h>
#include <ogdf/module/AugmentationModule.h>
#include <ogdf/module/ShellingOrderModule.h>


namespace ogdf {

	class ShellingOrder;

//---------------------------------------------------------
// Computation of a planar straight-line layout of a simple
// (no multi-edges, no self-loops) planar graph
//---------------------------------------------------------
class PlanarStraightLayout : public GridLayoutModule
{
public:
	PlanarStraightLayout();
	~PlanarStraightLayout() { }

	// options

	// optimize grid size
	bool sizeOptimization() const { return m_sizeOptimization; }
	void sizeOptimization(bool opt) { m_sizeOptimization = opt; }

	// (maximal) size of the base (size base = baseRatio*size(ext. face))
	double baseRatio() const { return m_baseRatio; }
	void baseRatio(double ratio) { m_baseRatio = ratio; }


	// module option for augmentation
	void setAugmenter(AugmentationModule *pAugmenter) {
		m_augmenter.set(pAugmenter);
	}
	
	// module option for shelling order
	void setShellingOrder(ShellingOrderModule *pOrder){
		m_computeOrder.set(pOrder);
	}


private:
	bool   m_sizeOptimization;
	double m_baseRatio;

	ModuleOption<AugmentationModule> m_augmenter;		//!< the augmentation module
	ModuleOption<ShellingOrderModule> m_computeOrder;	//!< the shelling order module

	// computes grid layout for graph G
	virtual void doCall(const Graph &G, GridLayout &gridLayout, IPoint &boundingBox);

	void computeCoordinates(const Graph &G,
		ShellingOrder &lmc,
		NodeArray<int> &x,
		NodeArray<int> &y);

};


} // end namespace ogdf


#endif
