/*
 * $Revision: 1.5 $
 * 
 * last checkin:
 *   $Author: kerkhof $ 
 *   $Date: 2007/06/14 19:35:23 $ 
 ***************************************************************/
 
/** \file
 * \brief Computes a connected subgraph G' of G containing node n.
 * 
 * \author Thorsten Kerkhof (Thorsten.Kerkhof@udo.edu)
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/

#include <ogdf/internal/planarity/ConnectedSubgraph.h>

namespace ogdf {

template<class T>
void ConnectedSubgraph<T>::recursion(Graph& SG,
                                     bool* nodeVisited,
                                     bool* edgeVisited,
                                     const node& nG,
                                     const NodeArray<T>& nodeLengthG,
                                     NodeArray<T>& nodeLengthSG,
                                     const EdgeArray<T>& edgeLengthG,
                                     EdgeArray<T>& edgeLengthSG,
                                     NodeArray<node>& nSG_to_nG,
                                     EdgeArray<edge>& eSG_to_eG,
                                     NodeArray<node>& nG_to_nSG,
                                     EdgeArray<edge>& eG_to_eSG)
{
	node nSG = SG.newNode();
	nodeLengthSG[nSG] = nodeLengthG[nG];
	nG_to_nSG[nG] = nSG;
  nSG_to_nG[nSG] = nG;
	nodeVisited[nG->index()] = true;

	edge eG;
	forall_adj_edges(eG, nG)
	{
		if (!nodeVisited[eG->source()->index()])
			recursion(SG, nodeVisited, edgeVisited, eG->source(),
			          nodeLengthG, nodeLengthSG, edgeLengthG, edgeLengthSG,
                nSG_to_nG, eSG_to_eG, nG_to_nSG, eG_to_eSG);
		else if (!nodeVisited[eG->target()->index()])
			recursion(SG, nodeVisited, edgeVisited, eG->target(),
			          nodeLengthG, nodeLengthSG, edgeLengthG, edgeLengthSG,
                nSG_to_nG, eSG_to_eG, nG_to_nSG, eG_to_eSG);
		if (!edgeVisited[eG->index()])
		{
			edge eSG = SG.newEdge(nG_to_nSG[eG->source()], nG_to_nSG[eG->target()]);
			edgeLengthSG[eSG] = edgeLengthG[eG];
			eG_to_eSG[eG] = eSG;
      eSG_to_eG[eSG] = eG;
			edgeVisited[eG->index()] = true;
		}
	}
}

template<class T>
void ConnectedSubgraph<T>::call(const Graph& G,
                                Graph& SG,
                                const node& nG,
                                const NodeArray<T>& nodeLengthG,
                                NodeArray<T>& nodeLengthSG,
                                NodeArray<node>& nG_to_nSG)
{
	node nSG;
	NodeArray<node> nSG_to_nG;
	EdgeArray<edge> eSG_to_eG;
	EdgeArray<edge> eG_to_eSG;
	EdgeArray<T> edgeLengthG(G, 1);
	EdgeArray<T> edgeLengthSG;
	call(G, SG, nG, nSG, nSG_to_nG, eSG_to_eG, nG_to_nSG, eG_to_eSG,
	     nodeLengthG, nodeLengthSG, edgeLengthG, edgeLengthSG);
}

template<class T>
void ConnectedSubgraph<T>::call(const Graph& G,
                                Graph& SG,
                                const node& nG,
                                node& nSG,
                                const NodeArray<T>& nodeLengthG,
                                NodeArray<T>& nodeLengthSG)
{
	EdgeArray<T> edgeLengthG(G, 1);
	EdgeArray<T> edgeLengthSG;
	call(G, SG, nG, nSG, nodeLengthG, nodeLengthSG, edgeLengthG, edgeLengthSG);
}

template<class T>
void ConnectedSubgraph<T>::call(const Graph& G,
                                Graph& SG,
                                const node& nG,
                                node& nSG,
                                const NodeArray<T>& nodeLengthG,
                                NodeArray<T>& nodeLengthSG,
                                const EdgeArray<T>& edgeLengthG,
                                EdgeArray<T>& edgeLengthSG)
{
	NodeArray<node> nSG_to_nG(SG);
  EdgeArray<edge> eSG_to_eG(SG);
  call(G, SG, nG, nSG, nSG_to_nG, eSG_to_eG, nodeLengthG,
       nodeLengthSG, edgeLengthG, edgeLengthSG);
}

template<class T>
void ConnectedSubgraph<T>::call(const Graph& G,
                                Graph& SG,
                                const node& nG,
                                const NodeArray<T>& nodeLengthG,
                                NodeArray<T>& nodeLengthSG)
{
	node nSG;
	call(G, SG, nG, nSG, nodeLengthG, nodeLengthSG);
}

template<class T>
void ConnectedSubgraph<T>::call(const Graph& G,
                                Graph& SG,
                                const node& nG,
                                const NodeArray<T>& nodeLengthG,
                                NodeArray<T>& nodeLengthSG,
                                const EdgeArray<T>& edgeLengthG,
                                EdgeArray<T>& edgeLengthSG)
{
	node nSG = 0;
	call(G, SG, nG, nSG, nodeLengthG, nodeLengthSG, edgeLengthG, edgeLengthSG);
}

template<class T>
void ConnectedSubgraph<T>::call(const Graph& G,
                                Graph& SG,
                                const node& nG,
                                NodeArray<node>& nSG_to_nG)
{
	NodeArray<T> nodeLengthG(G, 0);
	NodeArray<T> nodeLengthSG(SG);
	EdgeArray<T> edgeLengthG(G, 0);
	EdgeArray<T> edgeLengthSG(SG);
	node nSG;
	EdgeArray<edge> eSG_to_eG;
	call(G, SG, nG, nSG, nSG_to_nG, eSG_to_eG, nodeLengthG, nodeLengthSG, edgeLengthG, edgeLengthSG);
}

template<class T>
void ConnectedSubgraph<T>::call(const Graph& G,
                                Graph& SG,
                                const node& nG,
                                node& nSG,
                                NodeArray<node>& nSG_to_nG,
                                EdgeArray<edge>& eSG_to_eG,
                                const NodeArray<T>& nodeLengthG,
                                NodeArray<T>& nodeLengthSG,
                                const EdgeArray<T>& edgeLengthG,
                                EdgeArray<T>& edgeLengthSG)
{
	NodeArray<node> nG_to_nSG;
	EdgeArray<edge> eG_to_eSG;
	call(G, SG, nG, nSG, nSG_to_nG, eSG_to_eG, nG_to_nSG, eG_to_eSG, nodeLengthG,
	     nodeLengthSG, edgeLengthG, edgeLengthSG);
}

template<class T>
void ConnectedSubgraph<T>::call(const Graph& G,
                                Graph& SG,
                                const node& nG,
                                node& nSG,
                                NodeArray<node>& nSG_to_nG,
                                EdgeArray<edge>& eSG_to_eG,
                                NodeArray<node>& nG_to_nSG,
                                EdgeArray<edge>& eG_to_eSG,
                                const NodeArray<T>& nodeLengthG,
                                NodeArray<T>& nodeLengthSG,
                                const EdgeArray<T>& edgeLengthG,
                                EdgeArray<T>& edgeLengthSG)
{
  SG.clear();
	bool* nodeVisited = new bool[G.numberOfNodes()];
	bool* edgeVisited = new bool[G.numberOfEdges()];
	for (int i = 0; i < G.numberOfNodes(); i++)
		nodeVisited[i] = false;
	for (int i = 0; i < G.numberOfEdges(); i++)
		edgeVisited[i] = false;
	nSG_to_nG.init(SG);
	eSG_to_eG.init(SG);
	nodeLengthSG.init(SG);
	edgeLengthSG.init(SG);
	nG_to_nSG.init(G);
	eG_to_eSG.init(G);

	recursion(SG, nodeVisited, edgeVisited, nG,
            nodeLengthG, nodeLengthSG, edgeLengthG, edgeLengthSG,
						nSG_to_nG, eSG_to_eG, nG_to_nSG, eG_to_eSG);
	nSG = nG_to_nSG[nG];

	delete nodeVisited;
	delete edgeVisited;
}

template<class T>
void ConnectedSubgraph<T>::call(const Graph& G,
                                Graph& SG,
                                const node& nG,
                                NodeArray<node>& nSG_to_nG,
                                EdgeArray<edge>& eSG_to_eG,
                                NodeArray<node>& nG_to_nSG,
                                EdgeArray<edge>& eG_to_eSG)
{
	SG.clear();
	bool* nodeVisited = new bool[G.numberOfNodes()];
	bool* edgeVisited = new bool[G.numberOfEdges()];
	for (int i = 0; i < G.numberOfNodes(); i++)
		nodeVisited[i] = false;
	for (int i = 0; i < G.numberOfEdges(); i++)
		edgeVisited[i] = false;
	nSG_to_nG.init(SG);
	eSG_to_eG.init(SG);
	NodeArray<T> nodeLengthG(G, 0);
	NodeArray<T> nodeLengthSG(SG);
	EdgeArray<T> edgeLengthG(G, 1);
	EdgeArray<T> edgeLengthSG(SG);
	nG_to_nSG.init(G);
	eG_to_eSG.init(G);

	recursion(SG, nodeVisited, edgeVisited, nG,
            nodeLengthG, nodeLengthSG, edgeLengthG, edgeLengthSG,
            nSG_to_nG, eSG_to_eG, nG_to_nSG, eG_to_eSG);

	delete nodeVisited;
	delete edgeVisited;
}

} // end namespace ogdf
