/*
 * $Revision: 1.3 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2007/06/08 18:39:30 $ 
 ***************************************************************/
 
/** \file
 * \brief Implements class MMCrossingMinimizationModule.
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/

#include <ogdf/module/MMCrossingMinimizationModule.h>
#include <ogdf/basic/simple_graph_alg.h>


namespace ogdf {


Module::ReturnType MMCrossingMinimizationModule::call(
	const Graph &G,
	const List<node> &splittableNodes,
	int &cr)
{
	cr = 0; m_nodeSplits = 0; m_splittedNodes = 0;

	NodeArray<bool> splittable(G,false);
	ListConstIterator<node> itV;
	for(itV = splittableNodes.begin(); itV.valid(); ++itV)
		splittable[*itV] = true;

	EdgeArray<int> comp(G,-1);
	int c = biconnectedComponents(G, comp);

	Array<List<edge> > edges(c);
	edge e;
	forall_edges(e,G) {
		edges[comp[e]].pushBack(e);
	}

	NodeArray<node> map(G,0);

	for(int i = 0; i < c; ++i)
	{
		if(edges[i].size() < 9)
			continue;
			
		Graph B;
		List<node> nodes;
		List<node> splittableNodesB;

		ListConstIterator<edge> it;
		for(it = edges[i].begin(); it.valid(); ++it)
		{
			edge e = *it;
			node v = e->source(), w = e->target();

			if(map[v] == 0) {
				map[v] = B.newNode();
				nodes.pushBack(v);
				if(splittable[v])
					splittableNodesB.pushBack(map[v]);
			}
			if(map[w] == 0) {
				map[w] = B.newNode();
				nodes.pushBack(w);
				if(splittable[w])
					splittableNodesB.pushBack(map[w]);
			}

			B.newEdge(map[v],map[w]);
		}

		PlanRepExpansion PG(B,splittableNodesB);

		int crcc, numNS = 0, numSN = 0;
		ReturnType ret = doCall(PG,0,crcc,numNS,numSN);
		if(isSolution(ret) == false)
			return ret;
		cr += crcc;
		m_nodeSplits    += numNS;
		m_splittedNodes += numSN;

		ListConstIterator<node> itV;
		for(itV = nodes.begin(); itV.valid(); ++itV)
			map[*itV] = 0;
	}

	return retFeasible;
}


Module::ReturnType MMCrossingMinimizationModule::call(
	const Graph &G,
	int &cr)
{
	cr = 0; m_nodeSplits = 0; m_splittedNodes = 0;

	EdgeArray<int> comp(G,-1);
	int c = biconnectedComponents(G, comp);

	Array<List<edge> > edges(c);
	edge e;
	forall_edges(e,G) {
		edges[comp[e]].pushBack(e);
	}

	NodeArray<node> map(G,0);

	for(int i = 0; i < c; ++i)
	{
		if(edges[i].size() < 9)
			continue;
			
		Graph B;
		List<node> nodes;

		ListConstIterator<edge> it;
		for(it = edges[i].begin(); it.valid(); ++it)
		{
			edge e = *it;
			node v = e->source(), w = e->target();

			if(map[v] == 0) {
				map[v] = B.newNode();
				nodes.pushBack(v);
			}
			if(map[w] == 0) {
				map[w] = B.newNode();
				nodes.pushBack(w);
			}

			B.newEdge(map[v],map[w]);
		}

		PlanRepExpansion PG(B);

		int crcc, numNS = 0, numSN = 0;
		ReturnType ret = doCall(PG,0,crcc,numNS,numSN);
		if(isSolution(ret) == false)
			return ret;
		cr += crcc;
		m_nodeSplits    += numNS;
		m_splittedNodes += numSN;

		ListConstIterator<node> itV;
		for(itV = nodes.begin(); itV.valid(); ++itV)
			map[*itV] = 0;
	}

	return retFeasible;
}


} // namspace ogdf

