/*
 * $Revision: 1.4 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2005/12/20 20:40:34 $ 
 ***************************************************************/
 
/** \file
 * \brief Implementation of class MaximalPlanarSubgraphSimple.
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#include <ogdf/planarity/MaximalPlanarSubgraphSimple.h>
#include <ogdf/planarity/PlanarModule.h>


namespace ogdf {


Module::ReturnType MaximalPlanarSubgraphSimple::doCall(
	const Graph &G,
	const List<edge> &preferedEdges,
	List<edge> &delEdges,
	const EdgeArray<int>  *pCost,
	bool preferedImplyPlanar)
{
	delEdges.clear();

	Graph H;
	NodeArray<node> mapToH(G);

	node v;
	forall_nodes(v,G)
		mapToH[v] = H.newNode();

	EdgeArray<bool> visited(G,false);
	PlanarModule pm;

	ListConstIterator<edge> it;
	for(it = preferedEdges.begin(); it.valid(); ++it)
	{
		edge eG = *it;
		visited[eG] = true;

		edge eH = H.newEdge(mapToH[eG->source()],mapToH[eG->target()]);

		if (preferedImplyPlanar == false && pm.planarityTest(H) == false) {
			H.delEdge(eH);
			delEdges.pushBack(eG);
		}
	}

	edge eG;
	forall_edges(eG,G)
	{
		if(visited[eG] == true)
			continue;

		edge eH = H.newEdge(mapToH[eG->source()],mapToH[eG->target()]);

		if (pm.planarityTest(H) == false) {
			H.delEdge(eH);
			delEdges.pushBack(eG);
		}
	}
	
	return retFeasible;
}


} // end namespace ogdf
